package org.jiemamy.internal.serializer.stax;

import static org.hamcrest.CoreMatchers.instanceOf;
import static org.junit.Assert.assertThat;

import java.io.InputStream;
import java.util.List;

import javax.xml.stream.EventFilter;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.XMLEvent;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.seasar.framework.util.tiger.CollectionsUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.jiemamy.internal.serializer.stax.command.CloseCommand;
import org.jiemamy.internal.serializer.stax.command.ContentsCommand;
import org.jiemamy.internal.serializer.stax.command.EmptyElementCommand;
import org.jiemamy.internal.serializer.stax.command.OpenCommand;
import org.jiemamy.internal.serializer.stax.command.end.EndElementCommand;
import org.jiemamy.internal.serializer.stax.command.start.StartElementCommand;
import org.jiemamy.model.Artemis;
import org.jiemamy.model.Jiemamy;
import org.jiemamy.model.RootModel;

/**
 * TODO for daisuke
 * @author daisuke
 */
public class JiemamyCommandInputStreamTest {
	
	private static Logger logger = LoggerFactory.getLogger(JiemamyCommandInputStreamTest.class);
	

	/**
	 * setup
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		Jiemamy.init(Artemis.class);
	}
	
	/**
	 * teardown
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		Jiemamy.dispose();
	}
	
	/**
	 * TODO for daisuke
	 * @throws Exception
	 */
	@Test
	public void test01_fromRootModel() throws Exception {
		RootModel rootModel = Jiemamy.newRootModel();
		JiemamyCommandInputStream stream = new JiemamyCommandInputStreamImpl(rootModel);
		
		List<Class<? extends JiemamyCommand>> expectedClasses = CollectionsUtil.newArrayList();
		{
			expectedClasses.add(OpenCommand.class);
			expectedClasses.add(StartElementCommand.class);
			expectedClasses.add(EmptyElementCommand.class);
			expectedClasses.add(EndElementCommand.class);
			expectedClasses.add(CloseCommand.class);
		}
		
		check(stream, expectedClasses);
		
		stream.close();
	}
	
	/**
	 * TODO for daisuke
	 * @throws Exception
	 */
	@Test
	public void test02_fromXmlModel() throws Exception {
		InputStream in = JiemamyCommandInputStreamTest.class.getResourceAsStream("/minimal.xml");
		
		XMLInputFactory factory = XMLInputFactory.newInstance();
		factory.setProperty(XMLInputFactory.IS_NAMESPACE_AWARE, true);
		XMLEventReader xmlEventReader =
				factory.createFilteredReader(factory.createXMLEventReader(in), new EventFilter() {
					
					public boolean accept(XMLEvent event) {
						if (event.isCharacters() && event.asCharacters().isWhiteSpace()) {
							return false;
						}
						return true;
					}
					
				});
		JiemamyCommandInputStream stream = new XmlEventReaderAdapter(xmlEventReader);
		
		List<Class<? extends JiemamyCommand>> expectedClasses = CollectionsUtil.newArrayList();
		{
			// <?xml version="1.0" encoding="utf-8" ?>
			expectedClasses.add(OpenCommand.class);
			
			// <foo xmlns:prefix="uri_prifix" xmlns="uri_anon">
			expectedClasses.add(StartElementCommand.class);
			
			// <bar prefix:localName="piyo">
			expectedClasses.add(StartElementCommand.class);
			
			// value
			expectedClasses.add(ContentsCommand.class);
			
			// </bar>
			expectedClasses.add(EndElementCommand.class);
			
			// <bar />
			expectedClasses.add(StartElementCommand.class);
			expectedClasses.add(EndElementCommand.class);
			
			// </foo>
			expectedClasses.add(EndElementCommand.class);
			expectedClasses.add(CloseCommand.class);
		}
		
		check(stream, expectedClasses);
		
		stream.close();
	}
	
	/**
	 * TODO for daisuke
	 * @param stream
	 * @param expectedClasses
	 * @throws JiemamyCommandException
	 */
	private void check(JiemamyCommandInputStream stream, List<Class<? extends JiemamyCommand>> expectedClasses)
			throws JiemamyCommandException {
		int index = 0;
		while (stream.hasNext()) {
			JiemamyCommand command = stream.nextCommand();
			logger.info(command.toString());
			assertThat("index=" + index, command, instanceOf(expectedClasses.get(index)));
			index++;
		}
	}
}
