/*
 * Copyright 2007-2008 MIYAMOTO Daisuke, jiemamy.org and the Others.
 * Created on 2008/12/17
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.serializer.stax;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertThat;

import java.io.ByteArrayOutputStream;

import javanet.staxutils.IndentingXMLEventWriter;

import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLOutputFactory;

import org.apache.commons.lang.CharEncoding;
import org.apache.commons.lang.CharUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.jiemamy.internal.serializer.stax.command.CloseCommand;
import org.jiemamy.internal.serializer.stax.command.ContentsCommand;
import org.jiemamy.internal.serializer.stax.command.OpenCommand;
import org.jiemamy.internal.serializer.stax.command.end.DialectEndCommand;
import org.jiemamy.internal.serializer.stax.command.end.RootModelEndCommand;
import org.jiemamy.internal.serializer.stax.command.start.DialectStartCommand;
import org.jiemamy.internal.serializer.stax.command.start.RootModelStartCommand;
import org.jiemamy.model.Artemis;
import org.jiemamy.model.Jiemamy;
import org.jiemamy.model.RootModel;
import org.jiemamy.utils.UUIDUtil;

/**
 * TODO for daisuke
 * @author daisuke
 */
public class JiemamyCommandOutputStreamTest {
	
	private static Logger logger = LoggerFactory.getLogger(JiemamyCommandOutputStreamTest.class);
	

	/**
	 * setup
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		Jiemamy.init(Artemis.class);
	}
	
	/**
	 * teardown
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		Jiemamy.dispose();
	}
	
	/**
	 * TODO for daisuke
	 * @throws Exception
	 */
	@Test
	public void test01_toRootModel() throws Exception {
		JiemamyCommandOutputStream stream = new JiemamyCommandOutputStreamImpl();
		
		writeTestDataTo(stream);
		
		stream.close();
		
		RootModel rootModel = ((JiemamyCommandOutputStreamImpl) stream).getRootModel();
		assertThat(rootModel, is(notNullValue()));
		logger.info(rootModel.toString());
	}
	
	/**
	 * TODO for daisuke
	 * @throws Exception
	 */
	@Test
	public void test02_toXmlModel() throws Exception {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		
		XMLOutputFactory factory = XMLOutputFactory.newInstance();
		factory.setProperty(XMLOutputFactory.IS_REPAIRING_NAMESPACES, true);
		XMLEventWriter writer = factory.createXMLEventWriter(out);
		IndentingXMLEventWriter indentingWriter = new IndentingXMLEventWriter(writer);
		indentingWriter.setNewLine(String.valueOf(CharUtils.LF));
		JiemamyCommandOutputStream stream = new XmlEventWriterAdapter(indentingWriter);
		
		writeTestDataTo(stream);
		
		stream.close();
		String result = out.toString(CharEncoding.UTF_8);
		
		logger.info(result);
	}
	
	/**
	 * TODO for daisuke
	 * @param stream
	 * @throws JiemamyCommandException
	 */
	private void writeTestDataTo(JiemamyCommandOutputStream stream) throws JiemamyCommandException {
		stream.write(new OpenCommand());
		stream.write(new RootModelStartCommand(UUIDUtil.valueOfOrRandom("ffffffff-ffff-ffff-ffff-ffffffffffff")));
		stream.write(new DialectStartCommand());
		stream.write(new ContentsCommand("com.example.HogeDialect"));
		stream.write(new DialectEndCommand());
		stream.write(new RootModelEndCommand());
		stream.write(new CloseCommand());
	}
}
