/*
 * Copyright 2007-2008 MIYAMOTO Daisuke, jiemamy.org and the Others.
 * Created on 2008/12/16
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.serializer.stax;

import java.io.InputStream;
import java.io.StringWriter;

import javanet.staxutils.IndentingXMLEventWriter;

import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import com.bea.xml.stream.EventFactory;

import org.codehaus.stax2.XMLInputFactory2;
import org.codehaus.stax2.XMLOutputFactory2;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.jiemamy.serializer.SerializationTest;
import org.jiemamy.utils.StaxUtil;

/**
 * TODO for daisuke
 * @author daisuke
 */
public class StaxTest {
	
	private static Logger logger = LoggerFactory.getLogger(StaxTest.class);
	

	/**
	 * TODO for daisuke
	 * @throws Exception
	 */
	@Test
	public void test01_writeByStAX() throws Exception {
		StringWriter stringWriter = new StringWriter();
		XMLOutputFactory outFactory = XMLOutputFactory2.newInstance();
		XMLEventWriter writer = outFactory.createXMLEventWriter(stringWriter);
		IndentingXMLEventWriter indentingWriter = new IndentingXMLEventWriter(writer);
		indentingWriter.setNewLine("\n");
		writer = indentingWriter;
		
		XMLEventFactory eventFactory = EventFactory.newInstance();
		
		writer.add(eventFactory.createStartDocument());
		writer.add(eventFactory.createStartElement("", "", "foo"));
		writer.add(eventFactory.createStartElement("", "", "bar"));
		writer.add(eventFactory.createAttribute("prefix", "uri", "localName", "piyo"));
		writer.add(eventFactory.createCharacters("value"));
		writer.add(eventFactory.createEndElement("", "", "bar"));
		writer.add(eventFactory.createStartElement("", "", "bar"));
		writer.add(eventFactory.createEndElement("", "", "bar"));
		writer.add(eventFactory.createEndElement("", "", "foo"));
		writer.add(eventFactory.createEndDocument());
		writer.close();
		
		logger.info(stringWriter.toString());
	}
	
	/**
	 * TODO for daisuke
	 * @throws Exception
	 */
	@Test
	public void test02_readByStAX() throws Exception {
		InputStream in = SerializationTest.class.getResourceAsStream("/minimal.xml");
		XMLInputFactory inFactory = XMLInputFactory2.newInstance();
		inFactory.setProperty(XMLInputFactory.IS_NAMESPACE_AWARE, true);
		XMLEventReader reader = inFactory.createXMLEventReader(in);
		
		while (reader.hasNext()) {
			XMLEvent event = reader.nextEvent();
			if (event.isStartElement()) {
				StartElement startElement = event.asStartElement();
				logger.info(StaxUtil.toEventTypeEnum(event.getEventType()) + ": " + startElement.getName().toString());
				Attribute id = startElement.getAttributeByName(JiemamyXmlConstants.ID);
				if (id != null) {
					logger.info("id=" + id.getValue());
				}
			} else if (event.isEndElement()) {
				EndElement endElement = event.asEndElement();
				logger.info(StaxUtil.toEventTypeEnum(event.getEventType()) + ": " + endElement.getName().toString());
			}
		}
		reader.close();
	}
}
